/* Doors for Linux.
   Copyright (C) 1998 Jason Lango

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/*
**  test_info.c - Test door_info() system call.
**  Jason Lango <jal@cs.brown.edu>
*/

#include "test.h"

const char *test_description(void)
{
	return "door_info() call";
}

void sigcleanup(int signo)
{
	test_destroy_door();
	exit(0);
}

void server_proc(void *cookie, char *argp, size_t arg_size,
	door_desc_t *dp, size_t n_desc)
{
	test_error("server: unexpected call to server_proc\n");
}

int server_main(int num, int argc, char *argv[])
{
	int door;
	struct door_info info;

	signal(SIGINT, sigcleanup);
	signal(SIGTERM, sigcleanup);
	signal(SIGHUP, sigcleanup);

	door = test_create_door(server_proc);

	VERIFY ( door_info(door, &info) );
	test_verify_info(&info, server_proc);

	for (;;)
		pause();
	return 0;
}

int client_main(int num, int argc, char *argv[])
{
	int fd;
	struct door_info info;
	door_attr_t attr;
	door_attr_t all_attr;

	fd = test_open_door();

	VERIFY ( door_info(fd, &info) );
	test_verify_info(&info, server_proc);

	if (dts->dts_verbose) {
		printf("Door server pid %d, procedure %lx, cookie %lx,\n"
			"   uniquifier %lx\n",
			(int) info.di_target,
			(unsigned long) info.di_proc,
			(unsigned long) info.di_data,
			(unsigned long) info.di_uniquifier);
	}

	if (dts->dts_verbose) {
		attr = info.di_attributes;
		all_attr = (DOOR_LOCAL|DOOR_UNREF|DOOR_REVOKED|DOOR_PRIVATE);

		printf("   attributes:");
		if (attr & DOOR_LOCAL)
			printf(" LOCAL");
		if (attr & DOOR_UNREF)
			printf(" UNREF");
		if (attr & DOOR_REVOKED)
			printf(" REVOKED");
		if (attr & DOOR_PRIVATE)
			printf(" PRIVATE");
		if (!(attr & all_attr))
			printf(" <none>");
		printf("\n");
	}

	test_kill_servers();
	return 0;
}
