/* Doors for Linux.
   Copyright (C) 1998 Jason Lango

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/*
**  test_unref.c - Test DOOR_UNREF messages.
**  Jason Lango <jal@cs.brown.edu>
*/

#include "test.h"

int unrefs_received = 0;
int unrefs_expected = 1;

void sigcleanup(int signo)
{
	if (unrefs_received == unrefs_expected)
		exit(0);
	else {
		test_error("server: received %d unref messages; expecting "
			"%d messages\n",
			unrefs_received, unrefs_expected);
		exit(1);
	}
}

void server_proc(void *cookie, char *argp, size_t arg_size,
	door_desc_t *dp, size_t n_desc)
{
	if (cookie != TEST_COOKIE) {
		test_error("server: cookie %lx != %lx\n",
			(long) cookie, (long) TEST_COOKIE);
	}

	if (arg_size || dp || n_desc) {
		test_error("server_proc: received unexpected args\n");
	}

	if (argp != DOOR_UNREF_DATA) {
		test_error("server_proc: argp %lx != DOOR_UNREF_DATA\n",
			(unsigned long) argp);
	}

	if (dts->dts_verbose) {
		printf("server: Received DOOR_UNREF_DATA message.\n");
	}

	++unrefs_received;

	VERIFY ( door_return(NULL, 0, NULL, 0) );
	test_error("door_return() returned\n");
}

const char *test_description(void)
{
	return "DOOR_UNREF messages";
}

int server_main(int num, int argc, char *argv[])
{
	int door;
	int fd;

	signal(SIGINT, sigcleanup);
	signal(SIGTERM, sigcleanup);
	signal(SIGHUP, sigcleanup);

	VERIFY ( door = door_create(server_proc, (void*) TEST_COOKIE,
		DOOR_UNREF) );
	VERIFY ( fd = creat(test_door_filename(), 0600) );
	VERIFY ( close(fd) );
	VERIFY ( fattach(door, test_door_filename()) );

	for (;;)
		pause();

	return 0;
}

int client_main(int num, int argc, char *argv[])
{
	int fd;

	fd = test_open_door();
	VERIFY ( close(fd) );

	/* At this point, there are two references to the door, one
	 * being the fattached file, the other being the open file
	 * descriptor held by the server.
	 */

	VERIFY ( fdetach(test_door_filename()) );

	/* The unref message should have been sent as a result of the
	 * fdetach.
	 */

	VERIFY ( unlink(test_door_filename()) );

	/* Possible race here, since the unref message is sent
	 * asynchronously w.r.t. the client process.
	 */
	sleep(1); /* Ick. */

	test_kill_servers();
	return 0;
}
